@@ -49,10 +49,6 @@ enum {
 
 static void         language_constructed       (GObject       *object);
 static void         language_finalize          (GObject       *object);
-static void         language_get_property      (GObject       *object,
-                                                guint          param_id,
-                                                GValue        *value,
-                                                GParamSpec    *pspec);
 static void         language_set_property      (GObject       *object,
                                                 guint          param_id,
                                                 const GValue  *value,
@@ -67,7 +63,6 @@ tracker_language_class_init (TrackerLanguageClass *klass)
 
 	object_class->constructed = language_constructed;
 	object_class->finalize = language_finalize;
-	object_class->get_property = language_get_property;
 	object_class->set_property = language_set_property;
 
 	g_object_class_install_property (object_class,
@@ -77,7 +72,7 @@ tracker_language_class_init (TrackerLanguageClass *klass)
 	                                                      "Language code",
 	                                                      NULL,
 	                                                      G_PARAM_WRITABLE |
-							      G_PARAM_CONSTRUCT_ONLY));
+	                                                      G_PARAM_CONSTRUCT_ONLY));
 }
 
 static void
@@ -106,27 +101,6 @@ language_finalize (GObject *object)
 	(G_OBJECT_CLASS (tracker_language_parent_class)->finalize) (object);
 }
 
-static void
-language_get_property (GObject    *object,
-                       guint       param_id,
-                       GValue     *value,
-                       GParamSpec *pspec)
-{
-	TrackerLanguagePrivate *priv;
-
-	priv = tracker_language_get_instance_private (TRACKER_LANGUAGE (object));
-
-	switch (param_id) {
-	case PROP_LANGUAGE_CODE:
-		g_value_set_string (value, priv->language_code);
-		break;
-
-	default:
-		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
-		break;
-	};
-}
-
 static void
 language_set_property (GObject      *object,
                        guint         param_id,
@@ -225,33 +199,20 @@ tracker_language_new (const gchar *language_code)
 	return language;
 }
 
-/**
- * tracker_language_stem_word:
- * @language: a #TrackerLanguage
- * @word: string pointing to a word
- * @word_length: word ascii length
- *
- * If the stemmer is enabled, it will return the stem word for @word.
- * If it's disabled, it will return the passed word.
- *
- * Returns: a string with the processed word. This string must be
- *          freed with g_free()
- **/
-gchar *
+void
 tracker_language_stem_word (TrackerLanguage *language,
-                            const gchar     *word,
-                            gint             word_length)
+                            gchar           *buffer,
+                            gint            *buffer_len,
+                            gint             buffer_size)
 {
 #ifdef HAVE_LIBSTEMMER
 	TrackerLanguagePrivate *priv;
 #endif /* HAVE_LIBSTEMMER */
-	gchar *stem_word = NULL;
 
-	g_return_val_if_fail (TRACKER_IS_LANGUAGE (language), NULL);
-
-	if (word_length < 0) {
-		word_length = strlen (word);
-	}
+	g_return_if_fail (TRACKER_IS_LANGUAGE (language));
+	g_return_if_fail (buffer != NULL);
+	g_return_if_fail (buffer_len != NULL);
+	g_return_if_fail (*buffer_len >= 0);
 
 #ifdef HAVE_LIBSTEMMER
 	priv = tracker_language_get_instance_private (language);
@@ -259,13 +220,20 @@ tracker_language_stem_word (TrackerLanguage *language,
 	g_mutex_lock (&priv->stemmer_mutex);
 
 	if (priv->stemmer) {
-		stem_word = g_strdup ((gchar *) sb_stemmer_stem (priv->stemmer,
-		                                                 (guchar*) word,
-		                                                 word_length));
+		const sb_symbol *symbol;
+		int len;
+
+		symbol = sb_stemmer_stem (priv->stemmer,
+		                          (const sb_symbol *) buffer,
+		                          *buffer_len);
+		len = sb_stemmer_length (priv->stemmer);
+
+		if (len < buffer_size) {
+			memcpy (buffer, symbol, len + 1);
+			*buffer_len = len;
+		}
 	}
 
 	g_mutex_unlock (&priv->stemmer_mutex);
 #endif /* HAVE_LIBSTEMMER */
-
-	return stem_word ? stem_word : g_strndup (word, word_length);
 }
